@@ -30,6 +30,7 @@
 #include "tool_libinfo.h"
 #include "tool_util.h"
 #include "tool_version.h"
+#include "tool_cb_prg.h"
 
 #include "memdebug.h" /* keep this as LAST include */
 
@@ -66,6 +67,7 @@ static const struct category_descriptors categories[] = {
   {"telnet", "TELNET protocol options", CURLHELP_TELNET},
   {"tftp", "TFTP protocol options", CURLHELP_TFTP},
   {"tls", "All TLS/SSL related options", CURLHELP_TLS},
+  {"ech", "All Encrypted Client Hello (ECH) options", CURLHELP_ECH},
   {"upload", "All options for uploads",
    CURLHELP_UPLOAD},
   {"verbose", "Options related to any kind of command line output of curl",
@@ -73,8 +75,7 @@ static const struct category_descriptors categories[] = {
   {NULL, NULL, CURLHELP_HIDDEN}
 };
 
-
-static void print_category(curlhelp_t category)
+static void print_category(curlhelp_t category, unsigned int cols)
 {
   unsigned int i;
   size_t longopt = 5;
@@ -91,23 +92,31 @@ static void print_category(curlhelp_t category)
     if(len > longdesc)
       longdesc = len;
   }
-  if(longopt + longdesc > 80)
-    longopt = 80 - longdesc;
+  if(longopt + longdesc > cols)
+    longopt = cols - longdesc;
 
   for(i = 0; helptext[i].opt; ++i)
     if(helptext[i].categories & category) {
-      printf(" %-*s %s\n", (int)longopt, helptext[i].opt, helptext[i].desc);
+      size_t opt = longopt;
+      size_t desclen = strlen(helptext[i].desc);
+      if(opt + desclen >= (cols - 2)) {
+        if(desclen < (cols - 2))
+          opt = (cols - 3) - desclen;
+        else
+          opt = 0;
+      }
+      printf(" %-*s  %s\n", (int)opt, helptext[i].opt, helptext[i].desc);
     }
 }
 
 /* Prints category if found. If not, it returns 1 */
-static int get_category_content(const char *category)
+static int get_category_content(const char *category, unsigned int cols)
 {
   unsigned int i;
   for(i = 0; categories[i].opt; ++i)
     if(curl_strequal(categories[i].opt, category)) {
       printf("%s: %s\n", categories[i].opt, categories[i].desc);
-      print_category(categories[i].category);
+      print_category(categories[i].category, cols);
       return 0;
     }
   return 1;
@@ -124,6 +133,7 @@ static void get_categories(void)
 
 void tool_help(char *category)
 {
+  unsigned int cols = get_terminal_columns();
   puts("Usage: curl [options...] <url>");
   /* If no category was provided */
   if(!category) {
@@ -131,18 +141,18 @@ void tool_help(char *category)
       "menu is stripped into categories.\nUse \"--help category\" to get "
       "an overview of all categories.\nFor all options use the manual"
       " or \"--help all\".";
-    print_category(CURLHELP_IMPORTANT);
+    print_category(CURLHELP_IMPORTANT, cols);
     puts(category_note);
   }
   /* Lets print everything if "all" was provided */
   else if(curl_strequal(category, "all"))
     /* Print everything except hidden */
-    print_category(~(CURLHELP_HIDDEN));
+    print_category(~(CURLHELP_HIDDEN), cols);
   /* Lets handle the string "category" differently to not print an errormsg */
   else if(curl_strequal(category, "category"))
     get_categories();
   /* Otherwise print category and handle the case if the cat was not found */
-  else if(get_category_content(category)) {
+  else if(get_category_content(category, cols)) {
     puts("Invalid category provided, here is a list of all categories:\n");
     get_categories();
   }
@@ -173,12 +183,30 @@ void tool_version_info(void)
   printf("Release-Date: %s\n", LIBCURL_TIMESTAMP);
 #endif
   if(built_in_protos[0]) {
+    const char *insert = NULL;
+    /* we have ipfs and ipns support if libcurl has http support */
+    for(builtin = built_in_protos; *builtin; ++builtin) {
+      if(insert) {
+        /* update insertion so ipfs will be printed in alphabetical order */
+        if(strcmp(*builtin, "ipfs") < 0)
+          insert = *builtin;
+        else
+          break;
+      }
+      else if(!strcmp(*builtin, "http")) {
+        insert = *builtin;
+      }
+    }
     printf("Protocols:");
     for(builtin = built_in_protos; *builtin; ++builtin) {
       /* Special case: do not list rtmp?* protocols.
          They may only appear together with "rtmp" */
       if(!curl_strnequal(*builtin, "rtmp", 4) || !builtin[0][4])
         printf(" %s", *builtin);
+      if(insert && insert == *builtin) {
+        printf(" ipfs ipns");
+        insert = NULL;
+      }
     }
     puts(""); /* newline */
   }
